package jalview.ext.archaeopteryx;

import jalview.datamodel.ColumnSelection;
import jalview.datamodel.HiddenColumns;
import jalview.datamodel.SequenceGroup;
import jalview.datamodel.SequenceI;
import jalview.gui.PaintRefresher;
import jalview.structure.SelectionSource;
import jalview.structure.StructureSelectionManager;
import jalview.viewmodel.AlignmentViewport;

import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.util.HashSet;
import java.util.List;
import java.util.Map;

import org.forester.archaeopteryx.MainFrame;
import org.forester.phylogeny.PhylogenyMethods;
import org.forester.phylogeny.PhylogenyNode;

/**
 * Class for binding the Archaeopteryx tree viewer to the Jalview alignment that
 * it originates from, meaning that selecting sequences in the tree viewer also
 * selects them in the alignment view and vice versa.
 * 
 * @author kjvanderheide
 *
 */
public final class AptxBinding
        implements ExternalTreeViewerBindingI<PhylogenyNode>
{
  private org.forester.archaeopteryx.TreePanel treeView;

  private AlignmentViewport parentAvport;

  private final StructureSelectionManager ssm;

  private Map<SequenceI, PhylogenyNode> sequencesBoundToNodes;

  private Map<PhylogenyNode, SequenceI> nodesBoundToSequences;

  /**
   * 
   * @param archaeopteryx
   * 
   * @param jalviewAlignmentViewport
   *          alignment viewport from which the tree was calculated.
   * 
   * @param alignMappedToNodes
   *          map with sequences used to calculate the tree and matching tree
   *          nodes as key, value pair respectively.
   * 
   * @param nodesMappedToAlign
   *          map with tree nodes and matching sequences used to calculate the
   *          tree as key, value pair respectively.
   */
  public AptxBinding(final MainFrame archaeopteryx,
          final AlignmentViewport jalviewAlignmentViewport,
          final Map<SequenceI, PhylogenyNode> alignMappedToNodes,
          final Map<PhylogenyNode, SequenceI> nodesMappedToAlign)
  {
    // deal with/prohibit null values here as that will cause problems
    parentAvport = jalviewAlignmentViewport;
    sequencesBoundToNodes = alignMappedToNodes;
    nodesBoundToSequences = nodesMappedToAlign;
    treeView = archaeopteryx.getMainPanel().getCurrentTreePanel();
    ssm = parentAvport.getStructureSelectionManager();
    ssm.addSelectionListener(this);
    treeView.addMouseListener(this);
    PaintRefresher.Register(treeView, parentAvport.getSequenceSetId());

  }

  @Override
  public void actionPerformed(ActionEvent e)
  {
  }

  @Override
  public void mouseClicked(MouseEvent e)
  {
  }

  @Override
  public void mousePressed(final MouseEvent e)
  {
    showNodeSelectionOnAlign(e);
  }

  @Override
  public void mouseReleased(MouseEvent e)
  {
  }

  @Override
  public void mouseEntered(MouseEvent e)
  {
  }

  @Override
  public void mouseExited(MouseEvent e)
  {
  }


  @Override
  public void selection(final SequenceGroup seqsel,
          final ColumnSelection colsel, final HiddenColumns hidden,
          final SelectionSource source)
  {
    if (source == parentAvport) // check if source is alignment from where the
    // tree originates
    {
      treeView.setFoundNodes0(
              new HashSet<Long>(seqsel.getSequences().size()));

      for (SequenceI selectedSequence : seqsel.getSequences())
      {
        PhylogenyNode matchingNode = sequencesBoundToNodes.get(selectedSequence);
        if (matchingNode != null)
        {
          treeView.getFoundNodes0().add(matchingNode.getId());
        }

      }
      PaintRefresher.Refresh(treeView, parentAvport.getSequenceSetId());

    }


  }

  /**
   * If a node is selected in the tree panel this method highlights the
   * corresponding sequence in the Jalview alignment view. If an internal node
   * is selected all child sequences get highlighted as well.
   */
  @Override
  public void showNodeSelectionOnAlign(final MouseEvent e)
  {
    final PhylogenyNode node = treeView.findNode(e.getX(), e.getY());
    if (node != null)
    {
      if ((e.getModifiers() & InputEvent.SHIFT_MASK) == 0) // clear previous
      // selection if shift
      // IS NOT pressed
      {
        parentAvport.setSelectionGroup(null);
      }

      if (node.isInternal())
      {
        showMatchingChildSequences(node);

      }
      else
      {
        showMatchingSequence(node);

      }

    }
  }




  @Override
  public void showMatchingSequence(final PhylogenyNode nodeToMatch)
  {
    SequenceI matchingSequence = nodesBoundToSequences.get(nodeToMatch);
    if (matchingSequence != null)
    {
      treeSelectionChanged(matchingSequence);
      parentAvport.sendSelection();
      PaintRefresher.Refresh(treeView, parentAvport.getSequenceSetId());
    }
  }

  @Override
  public void showMatchingChildSequences(final PhylogenyNode parentNode)
  {
    final List<PhylogenyNode> childNodes = PhylogenyMethods
            .getAllDescendants(parentNode);

    for (PhylogenyNode childNode : childNodes)
    {
      SequenceI matchingSequence = nodesBoundToSequences.get(childNode);
      if (matchingSequence != null)
      {
        treeSelectionChanged(matchingSequence);
      }
    }

    parentAvport.sendSelection();
    PaintRefresher.Refresh(treeView, parentAvport.getSequenceSetId());
  }

  /**
   * Refactored from TreeCanvas.
   * 
   * @param sequence
   *          of the node selected in the tree viewer.
   */
  @Override
  public void treeSelectionChanged(final SequenceI sequence)
  {
    SequenceGroup selected = parentAvport.getSelectionGroup();

    if (selected == null)
    {
      selected = new SequenceGroup();
      parentAvport.setSelectionGroup(selected);
    }

    selected.setEndRes(parentAvport.getAlignment().getWidth() - 1);
    selected.addOrRemove(sequence, true);

  }

  public AlignmentViewport getParentAvport()
  {
    return parentAvport;
  }

  public void setParentAvport(final AlignmentViewport parentAvport)
  {
    this.parentAvport = parentAvport;
  }

}



